unit uConfirmations;
{$INCLUDE 'DirSync3.inc'}

interface

uses
  SysUtils, Windows, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uSystem,
  uDirSyncConsts, uMessages, uOptions, uSupportFunctions, uCompatibility;

type
  TConfirmationDialogButton = (ocbYes, ocbYesToAll, ocbNo, ocbNoToAll, ocbCancel);
  TWhatToDoWithDifferentFilesDialogButton = (dfbOverwriteLeft, dfbAlwaysOverwriteLeft, dfbOverwriteRight, dfbAlwaysOverwriteRight, dfbSkip, dfbAlwaysSkip, dfbCancel);
  TControlInputInfo = {$IFDEF FAR3_UP} TInputRecord {$ELSE} LONG_PTR {$ENDIF} ;

  TConfirmationDialog = class(TButtonsDialog)
  private
    fRememberDirectory: string;
    fRememberActive: boolean;
    fGlobalRemember: boolean;
    fOnCompare: TNotifyEvent;
    fLabelTitle: TFarDialog.TLabel;
    fSeparator1: TFarDialog.TSeparator;
    fCheckRemember: TFarDialog.TCheckbox;
    fEditRememberDirectory: TFarDialog.TEdit;
    fButtonCompare: TFarDialog.TButton;
    fButtonCancel: TFarDialog.TButton;
    function GetTitle: string;
    procedure SetTitle(const Value: string);
    function GetRememberDirectory: string;
    procedure SetRememberDirectory(const Value: string);
    procedure SetGlobalRemember(const Value: boolean);
  protected
    procedure CreateUserControls; override;
    procedure CreateButtonControls; override;
    procedure Clear; override;
    procedure SetSize(Width, Height: integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    function Execute(out Button: TFarDialog.TButton): boolean;
    procedure ButtonCompareClick(Sender: TFarDialog.TButton; var StopProcessing: boolean);
    procedure CheckRememberClick(Sender: TFarDialog.TCheckbox; FinalState: TFarDialog.TCheckbox.TCheckState; var AllowChange: boolean);
    procedure CheckRememberKeyPress(Sender: TFarDialog.TControl; Key, ControlState: DWORD; var Handled: Boolean);
    function GetFileInfo(MessageID: TMessages; const FileName: string; Width: integer): string;
    property Title: string read GetTitle write SetTitle;
    property GlobalRemember: boolean read fGlobalRemember write SetGlobalRemember;
    property RememberDirectory: string read GetRememberDirectory write SetRememberDirectory;
    property RememberActive: boolean read fRememberActive;
    property OnCompare: TNotifyEvent read fOnCompare write fOnCompare;
    property LabelTitle: TFarDialog.TLabel read fLabelTitle;
    property Separator1: TFarDialog.TSeparator read fSeparator1;
    property CheckRemember: TFarDialog.TCheckbox read fCheckRemember;
    property EditRememberDirectory: TFarDialog.TEdit read fEditRememberDirectory;
    property ButtonCompare: TFarDialog.TButton read fButtonCompare;
    property ButtonCancel: TFarDialog.TButton read fButtonCancel;
  public
    constructor Create(GUID: TGUID);
    destructor Destroy; override;
  end;

  TTwoButtonConfirmationDialog = class(TConfirmationDialog)
  private
    fButtonYes: TFarDialog.TButton;
    fButtonNo: TFarDialog.TButton;
    fLabelDescription: TFarDialog.TLabel;
    fSeparator3: TFarDialog.TLabel;
    fLabelInfo: array of TFarDialog.TLabel;
    function GetLabelInfo(Index: integer): TFarDialog.TLabel;
  protected
    function GetLabelInfoCount: integer; virtual; abstract;
    procedure SetupDynamic; virtual; abstract;
    procedure CreateButtonControls; override;
    procedure CreateUserControls; override;
    procedure Clear; override;
    procedure SetSize(Width, Height: integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    property ButtonYes: TFarDialog.TButton read fButtonYes;
    property ButtonNo: TFarDialog.TButton read fButtonNo;
    property LabelDescription: TFarDialog.TLabel read fLabelDescription;
    property Separator3: TFarDialog.TLabel read fSeparator3;
    property LabelInfo[Index: integer]: TFarDialog.TLabel read GetLabelInfo;
    property LabelInfoCount: integer read GetLabelInfoCount;
  public
    function Execute: TConfirmationDialogButton;
    destructor Destroy; override;
    property RememberDirectory;
    property GlobalRemember;
  end;

  TOverwriteConfirmationDialog = class(TTwoButtonConfirmationDialog)
  private
    fSourceFileName: string;
    fDestinationFileName: string;
    fFromRightToLeft: boolean;
    fReadOnlyDestination: boolean;
  protected
    function GetLabelInfoCount: integer; override;
    procedure SetupDynamic; override;
  public
    constructor Create;
    property FromRightToLeft: boolean read fFromRightToLeft write fFromRightToLeft;
    property ReadOnlyDestination: boolean read fReadOnlyDestination write fReadOnlyDestination;
    property SourceFileName: string read fSourceFileName write fSourceFileName;
    property DestinationFileName: string read fDestinationFileName write fDestinationFileName;
    property OnCompare;
  end;

  TDeleteConfirmationDialog = class(TTwoButtonConfirmationDialog)
  private
    fFileName: string;
    fFileOnLeft: boolean;
    fReadOnlyFile: boolean;
    fIsDirectory: boolean;
  protected
    function GetLabelInfoCount: integer; override;
    procedure SetupDynamic; override;
  public
    constructor Create;
    property FileOnLeft: boolean read fFileOnLeft write fFileOnLeft;
    property ReadOnlyFile: boolean read fReadOnlyFile write fReadOnlyFile;
    property FileName: string read fFileName write fFileName;
    property IsDirectory: boolean read fIsDirectory write fIsDirectory;
  end;

  TRetrySkipCancelDialog = class(TTwoButtonConfirmationDialog)
  private
    fMessage1: TMessages;
    fFileName1: string;
  protected
    function GetLabelInfoCount: integer; override;
    procedure SetupDynamic; override;
  public
    constructor Create;
    property Message1: TMessages read fMessage1 write fMessage1;
    property FileName1: string read fFileName1 write fFileName1;
  end;

  TRetrySkipCancelDialog2 = class(TRetrySkipCancelDialog)
  private
    fMessage2: TMessages;
    fFileName2: string;
  protected
    function GetLabelInfoCount: integer; override;
    procedure SetupDynamic; override;
  public
    constructor Create;
    property Message2: TMessages read fMessage2 write fMessage2;
    property FileName2: string read fFileName2 write fFileName2;
  end;

  TWhatToDoWithDifferentFilesDialog = class(TConfirmationDialog)
  private
    fButtonOverwriteLeft: TFarDialog.TButton;
    fButtonOverwriteRight: TFarDialog.TButton;
    fButtonSkip: TFarDialog.TButton;
    fLabelDescription: TFarDialog.TLabel;
    fSeparator3: TFarDialog.TLabel;
    fLabelInfo: array of TFarDialog.TLabel;
    fRightFileName: string;
    fLeftFileName: string;
    fDisplayFileName: string;
    function GetLabelInfo(Index: integer): TFarDialog.TLabel;
  protected
    function GetLabelInfoCount: integer; virtual;
    procedure SetupDynamic; virtual;
    procedure CreateButtonControls; override;
    procedure CreateUserControls; override;
    procedure Clear; override;
    procedure SetSize(Width, Height: integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    property ButtonOverwriteLeft: TFarDialog.TButton read fButtonOverwriteLeft;
    property ButtonOverwriteRight: TFarDialog.TButton read fButtonOverwriteRight;
    property ButtonSkip: TFarDialog.TButton read fButtonSkip;
    property LabelDescription: TFarDialog.TLabel read fLabelDescription;
    property Separator3: TFarDialog.TLabel read fSeparator3;
    property LabelInfo[Index: integer]: TFarDialog.TLabel read GetLabelInfo;
    property LabelInfoCount: integer read GetLabelInfoCount;
  public
    constructor Create;
    function Execute: TWhatToDoWithDifferentFilesDialogButton;
    destructor Destroy; override;
    property RememberDirectory;
    property GlobalRemember;
    property LeftFileName: string read fLeftFileName write fLeftFileName;
    property RightFileName: string read fRightFileName write fRightFileName;
    property DisplayFileName: string read fDisplayFileName write fDisplayFileName;
    property OnCompare;
  end;

implementation

{ TBaseDialog }

procedure TConfirmationDialog.ButtonCompareClick(Sender: TFarDialog.TButton; var StopProcessing: boolean);
begin
  if Assigned(OnCompare) then
    OnCompare(Self);
  StopProcessing := True;
  ActiveControl := CheckRemember;
end;

procedure TConfirmationDialog.CheckRememberClick(Sender: TFarDialog.TCheckbox; FinalState: TFarDialog.TCheckbox.TCheckState; var AllowChange: boolean);
begin
  EditRememberDirectory.Enabled := (FinalState = csChecked) and EditRememberDirectory.Visible;
  EditRememberDirectory.Unchanged := False;
end;

procedure TConfirmationDialog.CheckRememberKeyPress(Sender: TFarDialog.TControl; Key, ControlState: DWORD; var Handled: Boolean);
const
  ControlKeyMask = {$IFDEF FAR3_UP} LEFT_ALT_PRESSED or LEFT_CTRL_PRESSED or RIGHT_ALT_PRESSED or RIGHT_CTRL_PRESSED or SHIFT_PRESSED {$ELSE} KEY_CTRLMASK {$ENDIF} ;
  {$IFDEF FAR3_UP}
  KEY_BS = VK_BACK;
  KEY_DEL = VK_DELETE;
  KEY_F5 = VK_F5;
  {$ENDIF}
begin
  if ActiveControl = CheckRemember then begin
    if (ControlState and ControlKeyMask) = 0 then begin
      case Key of
        KEY_BS:
          begin
            EditRememberDirectory.Text := RemoveLastPathComponent(EditRememberDirectory.Text);
            EditRememberDirectory.Unchanged := False;
            Handled := True;
          end;
        KEY_DEL:
          begin
            EditRememberDirectory.Text := '';
            EditRememberDirectory.Unchanged := False;
            Handled := True;
          end;
        KEY_F5:
          begin
            EditRememberDirectory.Text := RememberDirectory;
            EditRememberDirectory.Unchanged := False;
            Handled := True;
          end;
      end;
    end;
    Exit;
  end;
end;

procedure TConfirmationDialog.Clear;
begin
  inherited;
  fLabelTitle := nil;
  fSeparator1 := nil;
  fCheckRemember := nil;
  fEditRememberDirectory := nil;
  fButtonCompare := nil;
  fButtonCancel := nil;
end;

procedure TConfirmationDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
begin
  inherited;
  Height := 11;
end;

constructor TConfirmationDialog.Create(GUID: TGUID);
begin
  inherited;
  Flags := FMSG_WARNING;
end;

procedure TConfirmationDialog.CreateUserControls;
begin
  fLabelTitle := AddLabel(0, 0, 0, '', taCenter);
  fLabelTitle.Text := Self.Title;
  fSeparator1 := AddSeparator(0);
  fCheckRemember := AddCheckbox(0, 0, GetMsg(MRememberChoiceForDir), False);
  fCheckRemember.OnClick := CheckRememberClick;
  fCheckRemember.OnKeyPress := CheckRememberKeyPress;
  fEditRememberDirectory := AddEdit(0, 0, 0, '');
end;

procedure TConfirmationDialog.CreateButtonControls;
begin
  fButtonCompare := AddButton(0, 0, GetMsg(MCompare));
  fButtonCompare.NoClose := True;
  fButtonCompare.OnClick := ButtonCompareClick;
  fButtonCancel := AddButton(0, 0, GetMsg(MCancel));
end;

destructor TConfirmationDialog.Destroy;
begin
  inherited;
end;

function TConfirmationDialog.Execute(out Button: TFarDialog.TButton): boolean;
begin
  Result := False;
  fRememberActive := False;
  fCheckRemember.Checked := False;
  if GlobalRemember then
    fCheckRemember.Text := GetMsg(MRememberChoice)
  else
    fCheckRemember.Text := GetMsg(MRememberChoiceForDir);
  fEditRememberDirectory.Visible := not GlobalRemember;
  fEditRememberDirectory.Enabled := fCheckRemember.Checked and fEditRememberDirectory.Visible;
  fEditRememberDirectory.Text := RememberDirectory;
  fEditRememberDirectory.Unchanged := False;
  fButtonCompare.Visible := Assigned(fOnCompare);
  fButtonCompare.Enabled := fButtonCompare.Visible;
  if inherited Execute(Button) then
  begin
    fRememberActive := CheckRemember.Checked;
    if GlobalRemember then
      fRememberDirectory := ''
    else
      fRememberDirectory := ExcludeTrailingPathDelimiter(EditRememberDirectory.Text);
    Result := True;
  end;
end;

function TConfirmationDialog.GetFileInfo(MessageID: TMessages; const FileName: string; Width: integer): string;
var
  Attr: DWORD;
  Size: int64;
  CTime, MTime, ATime: TDateTime;
  sSize, sDate: string;
begin
  //Log('TConfirmationDialog.GetFileInfo');
  //Log('GetFileInfo for %s', [FileName]);
  Attr := GetFileAttributes(PChar(FileName));
  //Log('- Attributes = 0x%08.8x', [Attr]);
  if (Attr <> INVALID_FILE_ATTRIBUTES) and ((Attr and FILE_ATTRIBUTE_DIRECTORY) <> 0) then
    sSize := string(GetMsg(MDirectory))
  else begin
    Size := GetFileSize(FileName);
    //Log('- Size = %d', [Size]);
    if Size >= 1000000000000 then
      sSize := FormatCurr('###,###,##0" MiB"', Size div $100000)
    else if Size >= 0 then
      sSize := FormatCurr('###,###,###,##0', Size)
    else
      sSize := '?';
  end;
  if GetFileDateTime(FileName, CTime, MTime, ATime, True) then
    sDate := FormatDateTime('dd.mm.yyyy hh:nn:ss', MTime)
  else
    sDate := '?';
  Result := Format('%-*.*s   %15.15s %21.21s', [Width-40, Width-40, string(GetMsg(MessageID)), sSize, sDate]);
  //Log('TConfirmationDialog.GetFileInfo END');
end;

function TConfirmationDialog.GetRememberDirectory: string;
begin
  if GlobalRemember then
    Result := ''
  else
    Result := fRememberDirectory;
end;

function TConfirmationDialog.GetTitle: string;
begin
  Result := LabelTitle.Text;
end;

procedure TConfirmationDialog.SetSize(Width, Height: integer);
begin
  inherited;
  LabelTitle.AlignInside(Frame);
  ButtonCompare.Top := LabelButtonSeparator.Top + 1;
  ButtonCancel.Top := LabelButtonSeparator.Top + 1;
  CheckRemember.AlignBelow(LabelTitle);
  if GlobalRemember then begin
    CheckRemember.Top := LabelButtonSeparator.Top - 1;
    EditRememberDirectory.Visible := False;
  end
  else begin
    CheckRemember.Top := LabelButtonSeparator.Top - 2;
    EditRememberDirectory.Visible := True;
    EditRememberDirectory.AlignBelow(CheckRemember, 4);
  end;
  EditRememberDirectory.Enabled := CheckRemember.Checked and EditRememberDirectory.Visible;
  Separator1.Top := CheckRemember.Top - 1;
end;

procedure TConfirmationDialog.SetGlobalRemember(const Value: boolean);
begin
  if fGlobalRemember <> Value then begin
    fGlobalRemember := Value;
    if GlobalRemember then
      Height := Height - 1
    else
      Height := Height + 1;
  end;
end;

procedure TConfirmationDialog.SetRememberDirectory(const Value: string);
begin
  fRememberDirectory := ExcludeTrailingPathDelimiter(Value);
end;

procedure TConfirmationDialog.SetTitle(const Value: string);
begin
  LabelTitle.Text := Value;
end;

{ TTwoButtonConfirmationDialog }

procedure TTwoButtonConfirmationDialog.Clear;
begin
  inherited;
  fButtonYes := nil;
  fButtonNo := nil;
  fLabelDescription := nil;
  fSeparator3 := nil;
  SetLength(fLabelInfo, 0);
end;

procedure TTwoButtonConfirmationDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
var
  n: integer;
begin
  inherited;
  n := GetLabelInfoCount;
  if n > 0 then
    n := n + 1; // separator
  if not LabelDescription.Visible then
    n := n - 1;
  if not Separator3.Visible then
    n := n - 1;
  Height := Height + n;
end;

procedure TTwoButtonConfirmationDialog.CreateButtonControls;
begin
  fButtonYes := AddButton(0, 0, GetMsg(MYes));
  fButtonYes.Default := True;
  fButtonNo := AddButton(0, 0, GetMsg(MNo));
  inherited;
end;

procedure TTwoButtonConfirmationDialog.CreateUserControls;
var
  i, n: integer;
begin
  fLabelDescription := AddLabel(0, 0, 0, '');
  fLabelDescription.ShowAmpersand := True;
  n := LabelInfoCount;
  if n > 0 then begin
    fSeparator3 := AddSeparator(0);
    SetLength(fLabelInfo, n);
    for i := 0 to Pred(n) do begin
      fLabelInfo[i] := AddLabel(0, 0, 0, '');
      fLabelInfo[i].ShowAmpersand := True;
    end;
  end;
  inherited;
  LabelTitle.PlaceBefore(fLabelDescription);
end;

destructor TTwoButtonConfirmationDialog.Destroy;
begin
  inherited;
end;

function TTwoButtonConfirmationDialog.Execute: TConfirmationDialogButton;
var
  Button: TFarDialog.TButton;
  Remember: boolean;
begin
  Result := ocbCancel;
  SetupDynamic;
  if inherited Execute(Button) then begin
    Remember := Self.RememberActive;
    if Button = ButtonYes then
      if Remember then
        Result := ocbYesToAll
      else
        Result := ocbYes
    else if Button = ButtonNo then
      if Remember then
        Result := ocbNoToAll
      else
        Result := ocbNo;
  end;
end;

function TTwoButtonConfirmationDialog.GetLabelInfo(Index: integer): TFarDialog.TLabel;
begin
  if (Index < 0) or (Index >= Length(fLabelInfo)) then
    raise EListError.Create('List index out of bounds');
  Result := fLabelInfo[Index];
end;

procedure TTwoButtonConfirmationDialog.SetSize(Width, Height: integer);
var
  i, n, Delta: integer;
begin
  inherited;
  LabelDescription.AlignBelow(LabelTitle);
  n := LabelInfoCount;
  if n > 0 then begin
    Separator3.AlignBelow(LabelDescription);
    Delta := -1;
    if LabelDescription.Visible then
      Inc(Delta);
    if Separator3.Visible then
      Inc(Delta);
    for i := 0 to Pred(n) do begin
      LabelInfo[i].AlignBelow(LabelDescription, 0, i+Delta);
    end;
  end;
  ButtonYes.Top := ButtonCancel.Top;
  ButtonNo.Top := ButtonYes.Top;
  SetupDynamic;
end;

{ TOverwriteConfirmationDialog }

constructor TOverwriteConfirmationDialog.Create;
begin
  inherited Create(DIALOG_OVERWRITE_GUID);
  ButtonYes.Text := GetMsg(MOverwrite);
  ButtonNo.Text := GetMsg(MSkip);
  fFromRightToLeft := False;
  fReadOnlyDestination := False;
end;

function TOverwriteConfirmationDialog.GetLabelInfoCount: integer;
begin
  Result := 2;
end;

procedure TOverwriteConfirmationDialog.SetupDynamic;
begin
  inherited;
  if ReadOnlyDestination then
    if FromRightToLeft then
      Title := GetMsg(MLeftFileReadonly)
    else
      Title := GetMsg(MRightFileReadonly)
  else
    if FromRightToLeft then
      Title := GetMsg(MLeftFileAlreadyExists)
    else
      Title := GetMsg(MRightFileAlreadyExists);
  LabelDescription.Text := Plugin.TruncateString(RemoveNTSchematicsFromFileName(DestinationFileName), LabelDescription.Width, ' ');
  LabelInfo[0].Text := GetFileInfo(MNew, SourceFileName, LabelInfo[0].Width);
  LabelInfo[1].Text := GetFileInfo(MExisting, DestinationFileName, LabelInfo[1].Width);
end;

{ TDeleteConfirmationDialog }

constructor TDeleteConfirmationDialog.Create;
begin
  inherited Create(DIALOG_DELETE_GUID);
  ButtonYes.Text := GetMsg(MErase);
  ButtonNo.Text := GetMsg(MSkip);
  fFileOnLeft := False;
  fReadOnlyFile := False;
  fIsDirectory := False;
end;

function TDeleteConfirmationDialog.GetLabelInfoCount: integer;
begin
  Result := 1;
end;

procedure TDeleteConfirmationDialog.SetupDynamic;
begin
  inherited;
  if ReadOnlyFile then
    if FileOnLeft then
      if IsDirectory then
        Title := GetMsg(MLeftDirReadonly)
      else
        Title := GetMsg(MLeftFileReadonly)
    else
      if IsDirectory then
        Title := GetMsg(MRightDirReadonly)
      else
        Title := GetMsg(MRightFileReadonly)
  else
    if FileOnLeft then
      if IsDirectory then
        Title := GetMsg(MLeftDirErase)
      else
        Title := GetMsg(MLeftFileErase)
    else
      if IsDirectory then
        Title := GetMsg(MRightDirErase)
      else
        Title := GetMsg(MRightFileErase);
  LabelDescription.Text := Plugin.TruncateString(RemoveNTSchematicsFromFileName(FileName), LabelDescription.Width, ' ');
  if IsDirectory then
    LabelInfo[0].Text := GetFileInfo(MEraseDir, FileName, LabelInfo[0].Width)
  else
    LabelInfo[0].Text := GetFileInfo(MEraseFile, FileName, LabelInfo[0].Width);
end;

{ TRetrySkipCancelDialog }

constructor TRetrySkipCancelDialog.Create;
begin
  inherited Create(DIALOG_RETRYSKIPCANCEL_GUID);
  ButtonYes.Text := GetMsg(MRetry);
  ButtonNo.Text := GetMsg(MSkip);
  LabelDescription.Visible := False;
  Separator3.Visible := False;
  fMessage1 := MNoLngStringDefined;
  fFileName1 := '';
end;

function TRetrySkipCancelDialog.GetLabelInfoCount: integer;
begin
  Result := 1;
end;

procedure TRetrySkipCancelDialog.SetupDynamic;
begin
  inherited;
  LabelTitle.TextAligment := taLeft;
  LabelTitle.Text := GetMsg(Message1);
  LabelInfo[0].Text := Plugin.TruncateString(RemoveNTSchematicsFromFileName(FileName1), LabelInfo[0].Width, ' ');
end;

{ TRetrySkipCancelDialog2 }

constructor TRetrySkipCancelDialog2.Create;
begin
  inherited Create;
  fMessage2 := MNoLngStringDefined;
  fFileName2 := '';
end;

function TRetrySkipCancelDialog2.GetLabelInfoCount: integer;
begin
  Result := 3;
end;

procedure TRetrySkipCancelDialog2.SetupDynamic;
begin
  inherited;
  LabelInfo[1].Text := GetMsg(Message2);
  LabelInfo[2].Text := Plugin.TruncateString(RemoveNTSchematicsFromFileName(FileName2), LabelInfo[2].Width, ' ');
end;

{ TWhatToDoWithDifferentFilesDialog }

procedure TWhatToDoWithDifferentFilesDialog.Clear;
begin
  inherited;
  fButtonOverwriteLeft := nil;
  fButtonOverwriteRight := nil;
  fButtonSkip := nil;
  fLabelDescription := nil;
  fSeparator3 := nil;
  SetLength(fLabelInfo, 0);
end;

procedure TWhatToDoWithDifferentFilesDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
var
  n: integer;
begin
  inherited;
  n := GetLabelInfoCount;
  if n > 0 then
    Height := Height + n + 1;
end;

constructor TWhatToDoWithDifferentFilesDialog.Create;
begin
  inherited Create(DIALOG_DIFFERENTFILES_GUID);
end;

procedure TWhatToDoWithDifferentFilesDialog.CreateButtonControls;
begin
  fButtonOverwriteLeft := AddButton(0, 0, GetMsg(MOverwriteLeft));
  fButtonOverwriteRight := AddButton(0, 0, GetMsg(MOverwriteRight));
  fButtonSkip := AddButton(0, 0, GetMsg(MSkip));
  fButtonSkip.Default := True;
  inherited;
end;

procedure TWhatToDoWithDifferentFilesDialog.CreateUserControls;
var
  i, n: integer;
begin
  fLabelDescription := AddLabel(0, 0, 0, '');
  fLabelDescription.ShowAmpersand := True;
  n := LabelInfoCount;
  if n > 0 then begin
    fSeparator3 := AddSeparator(0);
    SetLength(fLabelInfo, n);
    for i := 0 to Pred(n) do begin
      fLabelInfo[i] := AddLabel(0, 0, 0, '');
      fLabelInfo[i].ShowAmpersand := True;
    end;
  end;
  inherited;
  LabelTitle.PlaceBefore(fLabelDescription);
end;

destructor TWhatToDoWithDifferentFilesDialog.Destroy;
begin
  inherited;
end;

function TWhatToDoWithDifferentFilesDialog.Execute: TWhatToDoWithDifferentFilesDialogButton;
var
  Button: TFarDialog.TButton;
  Remember: boolean;
begin
  Result := dfbCancel;
  SetupDynamic;
  if inherited Execute(Button) then begin
    Remember := Self.RememberActive;
    if Button = ButtonOverwriteLeft then
      if Remember then
        Result := dfbAlwaysOverwriteLeft
      else
        Result := dfbOverwriteLeft
    else if Button = ButtonOverwriteRight then
      if Remember then
        Result := dfbAlwaysOverwriteRight
      else
        Result := dfbOverwriteRight
    else if Button = ButtonSkip then
      if Remember then
        Result := dfbAlwaysSkip
      else
        Result := dfbSkip;
  end;
end;

function TWhatToDoWithDifferentFilesDialog.GetLabelInfo(Index: integer): TFarDialog.TLabel;
begin
  if (Index < 0) or (Index >= Length(fLabelInfo)) then
    raise EListError.Create('List index out of bounds');
  Result := fLabelInfo[Index];
end;

function TWhatToDoWithDifferentFilesDialog.GetLabelInfoCount: integer;
begin
  Result := 2;
end;

procedure TWhatToDoWithDifferentFilesDialog.SetSize(Width, Height: integer);
var
  i, n: integer;
begin
  inherited;
  LabelDescription.AlignBelow(LabelTitle);
  n := LabelInfoCount;
  if n > 0 then begin
    Separator3.AlignBelow(LabelDescription);
    for i := 0 to Pred(n) do begin
      LabelInfo[i].AlignBelow(LabelDescription, 0, i+1);
    end;
  end;
  ButtonOverwriteLeft.Top := ButtonCancel.Top;
  ButtonOverwriteRight.Top := ButtonCancel.Top;
  ButtonSkip.Top := ButtonCancel.Top;
  SetupDynamic;
end;

procedure TWhatToDoWithDifferentFilesDialog.SetupDynamic;
begin
  inherited;
  Title := GetMsg(MFilesAreDifferent);
  LabelDescription.Text := Plugin.TruncateString(RemoveNTSchematicsFromFileName(DisplayFileName), LabelDescription.Width, ' ');
  LabelInfo[0].Text := GetFileInfo(MLeftFile, LeftFileName, LabelInfo[0].Width);
  LabelInfo[1].Text := GetFileInfo(MRightFile, RightFileName, LabelInfo[1].Width);
end;

end.
