unit uOptionsDialog;
{$INCLUDE 'DirSync3.inc'}

interface

uses
  SysUtils, Windows, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uDirSyncConsts, uMessages, uOptions;

type
  TOptionsDialog = class(TOkCancelDialog)
  private
    fOptions: TDirSyncOptions;
    fLabelCompareCommand: TFarDialog.TLabel;
    fEditCompareCommand: TFarDialog.TEdit;
    fLabelCompareCommandArguments: TFarDialog.TLabel;
    fEditCompareCommandArguments: TFarDialog.TEdit;
    fCheckCompareCommandAcceptsNTFileNames: TFarDialog.TCheckbox;
    fCheckShowDescription: TFarDialog.TCheckbox;
    fCheckSilentModeByDefault: TFarDialog.TCheckbox;
    fCheckGlobalRemember: TFarDialog.TCheckbox;
  protected
    procedure Clear; override;
    procedure CreateUserControls; override;
    procedure SetSize(Width, Height: integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    property LabelCompareCommand: TFarDialog.TLabel read fLabelCompareCommand;
    property EditCompareCommand: TFarDialog.TEdit read fEditCompareCommand;
    property LabelCompareCommandArguments: TFarDialog.TLabel read fLabelCompareCommandArguments;
    property EditCompareCommandArguments: TFarDialog.TEdit read fEditCompareCommandArguments;
    property CheckCompareCommandAcceptsNTFileNames: TFarDialog.TCheckbox read fCheckCompareCommandAcceptsNTFileNames;
    property CheckShowDescription: TFarDialog.TCheckbox read fCheckShowDescription;
    property CheckSilentModeByDefault: TFarDialog.TCheckbox read fCheckSilentModeByDefault;
    property CheckGlobalRemember: TFarDialog.TCheckbox read fCheckGlobalRemember;
  public
    constructor Create;
    destructor Destroy; override;
    function Execute: boolean;
    property Options: TDirSyncOptions read fOptions write fOptions;
  end;

implementation

{ TOptionsDialog }

procedure TOptionsDialog.Clear;
begin
  inherited;
  fLabelCompareCommand := nil;
  fEditCompareCommand := nil;
  fLabelCompareCommandArguments := nil;
  fEditCompareCommandArguments := nil;
  fCheckCompareCommandAcceptsNTFileNames := nil;
  fCheckShowDescription := nil;
  fCheckSilentModeByDefault := nil;
  fCheckGlobalRemember := nil;
end;

procedure TOptionsDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
begin
  inherited;
  Height := 14;
end;

constructor TOptionsDialog.Create;
begin
  inherited Create(DIALOG_CONFIG_GUID);
end;

procedure TOptionsDialog.CreateUserControls;
begin
  inherited;
  fLabelCompareCommand := AddLabel(0, 0, 0, GetMsg(MCompareCommand));
  fEditCompareCommand := AddEdit(0, 0, 0, '');
  fLabelCompareCommandArguments := AddLabel(0, 0, 0, GetMsg(MCompareCommandArguments));
  fEditCompareCommandArguments := AddEdit(0, 0, 0, '');
  fCheckCompareCommandAcceptsNTFileNames := AddCheckbox(0, 0, GetMsg(MCompareCommandAcceptsNTFileNames), False);
  fCheckShowDescription := AddCheckbox(0, 0, GetMsg(MShowDescriptionInSyncFile), True);
  fCheckSilentModeByDefault := AddCheckbox(0, 0, GetMsg(MUseSilentModeByDefault), False);
  fCheckGlobalRemember := AddCheckbox(0, 0, GetMsg(MGlobalRemember), False);
end;

destructor TOptionsDialog.Destroy;
begin
  inherited;
end;

function TOptionsDialog.Execute: boolean;
begin
  EditCompareCommand.Text := Options.CompareCommand;
  EditCompareCommandArguments.Text := Options.CompareCommandArguments;
  CheckCompareCommandAcceptsNTFileNames.Checked := Options.CompareCommandAcceptsNTFileNames;
  CheckSilentModeByDefault.Checked := Options.SilentModeByDefault;
  CheckShowDescription.Checked := Options.ShowDescriptionIsSyncFile;
  CheckGlobalRemember.Checked := Options.GlobalRemember;
  Result := False;
  if inherited Execute then
  begin
    Options.Load;
    Options.CompareCommand := EditCompareCommand.Text;
    Options.CompareCommandArguments := EditCompareCommandArguments.Text;
    Options.CompareCommandAcceptsNTFileNames := CheckCompareCommandAcceptsNTFileNames.Checked;
    Options.SilentModeByDefault := CheckSilentModeByDefault.Checked;
    Options.ShowDescriptionIsSyncFile := CheckShowDescription.Checked;
    Options.GlobalRemember := CheckGlobalRemember.Checked;
    Options.Save;
    Result := True;
  end;
end;

procedure TOptionsDialog.SetSize(Width, Height: integer);
begin
  inherited;
  LabelCompareCommand.AlignInside(Frame);
  EditCompareCommand.AlignBelow(LabelCompareCommand);
  LabelCompareCommandArguments.AlignBelow(EditCompareCommand);
  EditCompareCommandArguments.AlignBelow(LabelCompareCommandArguments);
  CheckCompareCommandAcceptsNTFileNames.AlignBelow(EditCompareCommandArguments);
  CheckShowDescription.AlignBelow(CheckCompareCommandAcceptsNTFileNames);
  CheckSilentModeByDefault.AlignBelow(CheckShowDescription);
  CheckGlobalRemember.AlignBelow(CheckSilentModeByDefault);
end;

end.
