unit uSynchronizeDirectoriesDialog;
{$INCLUDE 'DirSync3.inc'}

interface

uses
  SysUtils, Windows, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uSystem,
  uDirSyncConsts, uMessages, uOptions;

type

  TSynchronizeDirectoriesDialog = class(TOkCancelDialog)
  public
    type
      TDifferentFilesAction = (dfaAsk, dfaSkip, dfaCopyToLeft, dfaCopyToRight);
      TSynchronizeDirectoriesAction = (sdaSynchronize, sdaReturnToEditor, sdaCancel);
  private
    fCopyToLeftPanel: boolean;
    fConfirmOverwriteLeft: boolean;
    fCopyToRightPanel: boolean;
    fConfirmOverwriteRight: boolean;
    fDeleteLeftMissingOnRight: boolean;
    fDeleteRightMissingOnLeft: boolean;
    fDiffTypesPresent: array[TDifferenceType] of boolean;
    fDifferentFilesAction: TDifferentFilesAction;
    fSilentMode: boolean;
    fGlobalRemember: boolean;
    function GetDiffTypePresent(DiffType: TDifferenceType): boolean;
    procedure SetDiffTypePresent(DiffType: TDifferenceType; const Value: boolean);
  private
    fLabelBothFilesExist: TFarDialog.TLabel;
    fCheckCopyToLeft: TFarDialog.TCheckbox;
    fCheckCopyToLeftOverwrite: TFarDialog.TCheckbox;
    fCheckCopyToRight: TFarDialog.TCheckbox;
    fCheckCopyToRightOverwrite: TFarDialog.TCheckbox;
    fLabelOneFileDoesNotExist: TFarDialog.TLabel;
    fCheckDeleteLeft: TFarDialog.TCheckbox;
    fCheckDeleteRight: TFarDialog.TCheckbox;
    fLabelBothFilesExistWithSameDate: TFarDialog.TLabel;
    fRadioDifferentFilesAsk: TFarDialog.TRadioButton;
    fRadioDifferentFilesOverwriteLeft: TFarDialog.TRadioButton;
    fRadioDifferentFilesOverwriteRight: TFarDialog.TRadioButton;
    fRadioDifferentFilesSkip: TFarDialog.TRadioButton;
    fSeparator1: TFarDialog.TSeparator;
    fCheckSilentMode: TFarDialog.TCheckbox;
    fCheckGlobalRemember: TFarDialog.TCheckbox;
    fButtonEditor: TFarDialog.TButton;
  protected
    procedure CreateUserControls; override;
    procedure CreateButtonControls; override;
    procedure SetSize(Width, Height: Integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    procedure UpdateControls(ActiveCheckbox: TFarDialog.TCheckbox; ActiveCheckboxFinalState: TFarDialog.TCheckbox.TCheckState);
    property LabelBothFilesExist: TFarDialog.TLabel read fLabelBothFilesExist;
    property CheckCopyToLeft: TFarDialog.TCheckbox read fCheckCopyToLeft;
    property CheckCopyToLeftOverwrite: TFarDialog.TCheckbox read fCheckCopyToLeftOverwrite;
    property CheckCopyToRight: TFarDialog.TCheckbox read fCheckCopyToRight;
    property CheckCopyToRightOverwrite: TFarDialog.TCheckbox read fCheckCopyToRightOverwrite;
    property LabelOneFileDoesNotExist: TFarDialog.TLabel read fLabelOneFileDoesNotExist;
    property CheckDeleteLeft: TFarDialog.TCheckbox read fCheckDeleteLeft;
    property CheckDeleteRight: TFarDialog.TCheckbox read fCheckDeleteRight;
    property LabelBothFilesExistWithSameDate: TFarDialog.TLabel read fLabelBothFilesExistWithSameDate;
    property RadioDifferentFilesAsk: TFarDialog.TRadioButton read fRadioDifferentFilesAsk;
    property RadioDifferentFilesOverwriteLeft: TFarDialog.TRadioButton read fRadioDifferentFilesOverwriteLeft;
    property RadioDifferentFilesOverwriteRight: TFarDialog.TRadioButton read fRadioDifferentFilesOverwriteRight;
    property RadioDifferentFilesSkip: TFarDialog.TRadioButton read fRadioDifferentFilesSkip;
    property Separator1: TFarDialog.TSeparator read fSeparator1;
    property CheckSilentMode: TFarDialog.TCheckbox read fCheckSilentMode;
    property CheckGlobalRemember: TFarDialog.TCheckbox read fCheckGlobalRemember;
    property ButtonEditor: TFarDialog.TButton read fButtonEditor;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; override;
    function Execute: TSynchronizeDirectoriesAction;
    procedure SetDefaultSettings;
    property CopyToLeftPanel: boolean read fCopyToLeftPanel write fCopyToLeftPanel;
    property ConfirmOverwriteLeft: boolean read fConfirmOverwriteLeft write fConfirmOverwriteLeft;
    property CopyToRightPanel: boolean read fCopyToRightPanel write fCopyToRightPanel;
    property ConfirmOverwriteRight: boolean read fConfirmOverwriteRight write fConfirmOverwriteRight;
    property DeleteLeftMissingOnRight: boolean read fDeleteLeftMissingOnRight write fDeleteLeftMissingOnRight;
    property DeleteRightMissingOnLeft: boolean read fDeleteRightMissingOnLeft write fDeleteRightMissingOnLeft;
    property DifferentFilesAction: TDifferentFilesAction read fDifferentFilesAction write fDifferentFilesAction;
    property DiffTypePresent[DiffType: TDifferenceType]: boolean read GetDiffTypePresent write SetDiffTypePresent;
    property SilentMode: boolean read fSilentMode write fSilentMode;
    property GlobalRemember: boolean read fGlobalRemember write fGlobalRemember;
  end;

implementation

{ TDirSyncSyncDialog }

procedure TSynchronizeDirectoriesDialog.Clear;
begin
  inherited;
  fLabelBothFilesExist := nil;
  fCheckCopyToLeft := nil;
  fCheckCopyToLeftOverwrite := nil;
  fCheckCopyToRight := nil;
  fCheckCopyToRightOverwrite := nil;
  fLabelBothFilesExistWithSameDate := nil;
  fRadioDifferentFilesAsk := nil;
  fRadioDifferentFilesOverwriteLeft := nil;
  fRadioDifferentFilesOverwriteRight := nil;
  fRadioDifferentFilesSkip := nil;
  fLabelOneFileDoesNotExist := nil;
  fCheckDeleteLeft := nil;
  fCheckDeleteRight := nil;
  fSeparator1 := nil;
  fCheckSilentMode := nil;
  fCheckGlobalRemember := nil;
  fButtonEditor := nil;
end;

procedure TSynchronizeDirectoriesDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
begin
  inherited;
  Width := 76;
  Height := 19;
end;

constructor TSynchronizeDirectoriesDialog.Create;
begin
  inherited Create(DIALOG_MAIN_GUID);
  SetDefaultSettings;
end;

procedure TSynchronizeDirectoriesDialog.CreateButtonControls;
begin
  inherited;
  fButtonEditor := AddButton(0, 0, GetMsg(MReturnToEditor));
  fButtonEditor.PlaceAfter(ButtonOK);
end;

procedure TSynchronizeDirectoriesDialog.CreateUserControls;
begin
  inherited;
  fLabelBothFilesExist := AddLabel(0, 0, 0, GetMsg(MBothFilesExist));
  fCheckCopyToLeft := AddCheckbox(0, 0, GetMsg(MCopyRightToLeft), False);
  fCheckCopyToLeftOverwrite := AddCheckbox(0, 0, GetMsg(MOverwriteRightToLeft), False);
  fCheckCopyToRight := AddCheckbox(0, 0, GetMsg(MCopyLeftToRight), False);
  fCheckCopyToRightOverwrite := AddCheckbox(0, 0, GetMsg(MOverwriteLeftToRight), False);
  fLabelBothFilesExistWithSameDate := AddLabel(0, 0, 0, GetMsg(MBothFilesExistSameDate));
  fRadioDifferentFilesAsk := AddRadioButton(0, 0, GetMsg(MAskUser), True);
  fRadioDifferentFilesSkip := AddRadioButton(0, 0, GetMsg(MSkip), False);
  fRadioDifferentFilesOverwriteLeft := AddRadioButton(0, 0, GetMsg(MOverwriteLeft2), False);
  fRadioDifferentFilesOverwriteRight := AddRadioButton(0, 0, GetMsg(MOverwriteRight2), False);
  fLabelOneFileDoesNotExist := AddLabel(0, 0, 0, GetMsg(MEraseAbsentFiles));
  fCheckDeleteLeft := AddCheckbox(0, 0, GetMsg(MEraseAbsentFilesLeft), False);
  fCheckDeleteRight := AddCheckbox(0, 0, GetMsg(MEraseAbsentFilesRight), False);
  fSeparator1 := AddSeparator(0);
  fCheckSilentMode := AddCheckbox(0, 0, GetMsg(MSilentMode), False);
  fCheckGlobalRemember := AddCheckbox (0, 0, GetMsg(MGlobalRemember), False);
end;

destructor TSynchronizeDirectoriesDialog.Destroy;
begin
  inherited;
end;

function TSynchronizeDirectoriesDialog.Execute: TSynchronizeDirectoriesAction;
const
  EnabledFlag: array[boolean] of DWORD = (DIF_DISABLE, 0);
var
  Button: TFarDialog.TButton;
begin
  Result := sdaCancel;
  UpdateControls(nil, csUnchecked);
  CheckCopyToLeft.Checked := CopyToLeftPanel and CheckCopyToLeft.Enabled;
  CheckCopyToLeftOverwrite.Checked := ConfirmOverwriteLeft and CheckCopyToLeftOverwrite.Enabled;
  CheckCopyToRight.Checked := CopyToRightPanel and CheckCopyToRight.Enabled;
  CheckCopyToRightOverwrite.Checked := ConfirmOverwriteRight and CheckCopyToRightOverwrite.Enabled;
  RadioDifferentFilesAsk.Checked := True;
  CheckDeleteLeft.Checked := DeleteLeftMissingOnRight and CheckDeleteLeft.Enabled;
  CheckDeleteRight.Checked := DeleteRightMissingOnLeft and CheckDeleteRight.Enabled;
  CheckSilentMode.Checked := SilentMode;
  CheckGlobalRemember.Checked := GlobalRemember;
  if inherited Execute(Button) then
  begin
    if Button = ButtonOK then
    begin
      //Log('OK pressed, reading the settings');
      CopyToLeftPanel := CheckCopyToLeft.Checked;
      ConfirmOverwriteLeft := CheckCopyToLeftOverwrite.Checked;
      CopyToRightPanel := CheckCopyToRight.Checked;
      ConfirmOverwriteRight := CheckCopyToRightOverwrite.Checked;
      DeleteLeftMissingOnRight := CheckDeleteLeft.Checked;
      DeleteRightMissingOnLeft := CheckDeleteRight.Checked;
      if RadioDifferentFilesAsk.Checked then
        DifferentFilesAction := dfaAsk
      else if RadioDifferentFilesOverwriteLeft.Checked then
        DifferentFilesAction := dfaCopyToLeft
      else if RadioDifferentFilesOverwriteRight.Checked then
        DifferentFilesAction := dfaCopyToRight
      else if RadioDifferentFilesSkip.Checked then
        DifferentFilesAction := dfaSkip
      else
        DifferentFilesAction := dfaAsk;
      fSilentMode := CheckSilentMode.Checked;
      fGlobalRemember := CheckGlobalRemember.Checked;
      Result := sdaSynchronize;
    end
    else if Button = ButtonEditor then
    begin
      Result := sdaReturnToEditor;
    end;
  end;
end;

function TSynchronizeDirectoriesDialog.GetDiffTypePresent(DiffType: TDifferenceType): boolean;
begin
  Result := fDiffTypesPresent[DiffType];
end;

procedure TSynchronizeDirectoriesDialog.SetDefaultSettings;
begin
  CopyToLeftPanel := True;
  ConfirmOverwriteLeft := True;
  CopyToRightPanel := True;
  ConfirmOverwriteRight := True;
  DeleteLeftMissingOnRight := False;
  DeleteRightMissingOnLeft := False;
  DifferentFilesAction := dfaAsk;
  SilentMode := False;
  GlobalRemember := False;
end;

procedure TSynchronizeDirectoriesDialog.SetDiffTypePresent(DiffType: TDifferenceType; const Value: boolean);
begin
  fDiffTypesPresent[DiffType] := Value;
end;

procedure TSynchronizeDirectoriesDialog.SetSize(Width, Height: Integer);
begin
  inherited;
  LabelBothFilesExist.AlignInside(Frame);
  CheckCopyToLeft.AlignBelow(LabelBothFilesExist);
  CheckCopyToLeftOverwrite.AlignBelow(CheckCopyToLeft, 4);
  CheckCopyToRight.AlignBelow(CheckCopyToLeft, 0, 1);
  CheckCopyToRightOverwrite.AlignBelow(CheckCopyToRight, 4);
  LabelBothFilesExistWithSameDate.AlignBelow(CheckCopyToRight, 0, 1);
  RadioDifferentFilesAsk.AlignBelow(LabelBothFilesExistWithSameDate, 2);
  RadioDifferentFilesSkip.AlignNextTo(RadioDifferentFilesAsk);
  RadioDifferentFilesOverwriteLeft.AlignNextTo(RadioDifferentFilesSkip);
  RadioDifferentFilesOverwriteRight.AlignNextTo(RadioDifferentFilesOverwriteLeft);
  LabelOneFileDoesNotExist.AlignBelow(LabelBothFilesExistWithSameDate, 0, 1);
  CheckDeleteLeft.AlignBelow(LabelOneFileDoesNotExist);
  CheckDeleteRight.AlignBelow(CheckDeleteLeft);
  Separator1.AlignBelow(CheckDeleteRight);
  CheckSilentMode.AlignBelow(CheckDeleteRight, 0, 1);
  CheckGlobalRemember.AlignBelow(CheckSilentMode);
  ButtonEditor.Top := ButtonOK.Top;
end;

procedure TSynchronizeDirectoriesDialog.UpdateControls(ActiveCheckbox: TFarDialog.TCheckbox; ActiveCheckboxFinalState: TFarDialog.TCheckbox.TCheckState);
begin
  CheckCopyToLeft.Enabled := DiffTypePresent[dtMissingLeft] or DiffTypePresent[dtOlderLeft] or DiffTypePresent[dtFailedLeft];
  CheckCopyToLeftOverwrite.Enabled := CheckCopyToLeft.Enabled;
  CheckCopyToRight.Enabled := DiffTypePresent[dtMissingRight] or DiffTypePresent[dtOlderRight] or DiffTypePresent[dtFailedRight];
  CheckCopyToRightOverwrite.Enabled := CheckCopyToRight.Enabled;
  RadioDifferentFilesAsk.Enabled := DiffTypePresent[dtDifferent];
  RadioDifferentFilesSkip.Enabled := DiffTypePresent[dtDifferent];
  RadioDifferentFilesOverwriteLeft.Enabled := DiffTypePresent[dtDifferent];
  RadioDifferentFilesOverwriteRight.Enabled := DiffTypePresent[dtDifferent];
  CheckDeleteLeft.Enabled := DiffTypePresent[dtMissingRight];
  CheckDeleteRight.Enabled := DiffTypePresent[dtMissingLeft];
end;

end.
